#! /usr/bin/env python
'''Draws text to screen using WGL bitmap functionality

This code is a descendant of NeHe tutorial #13...

OpenGL Tutorial #13.

Project Name: Jeff Molofee's OpenGL Tutorial

Project Description: Using Bitmap Fonts In OpenGL

Authors Name: Jeff Molofee (aka NeHe)

Authors Web Site: nehe.gamedev.net

COPYRIGHT AND DISCLAIMER: (c)2000 Jeff Molofee

	If you plan to put this program on your web page or a cdrom of
	any sort, let me know via email, I'm curious to see where
	it ends up :)

        If you use the code for your own projects please give me credit,
        or mention my web site somewhere in your program or it's docs.
'''
from OpenGLContext.wxinteractivecontext import wxInteractiveContext
from OpenGL.GL import *
from math import cos, sin
import sys
try:
	from OpenGL.WGL import *
	import win32ui, win32con
except ImportError, err:
	raise ImportError( """Unable to import Win32 text modules: %s"""%(err,))

### eventually this should turn into a module under scenegraph/text
class font:
	def __init__(self, context):
		self.base = glGenLists(96)								# Storage For 96 Characters
		wgldc = wglGetCurrentDC()
		dc = win32ui.CreateDCFromHandle( wgldc )
		## pitch and family value
		f = win32ui.CreateFont(
			{
				'italic': None, #use of None is required, 0 doesn't work
				'underline': None, #use of None is required, 0 doesn't work
				'name': 'Times New Roman',
				'weight': 700,
				'height': 20,
			}
		)
		dc.SelectObject(
			f
		)
		wglUseFontBitmaps(wgldc, 32, 96, self.base)	# Builds 96 Characters Starting At Character 32
		# f is deleted here, if it is deleted before the display-lists
		# are created they will just use the default font!
	def __del__(self):
		glDeleteLists(self.base, 96)	# Delete All 96 Characters
	def draw(self, text):
		glCallLists(map(lambda x, y: ord(x) - 32 + y, text, [self.base]*len(text)))		# Draws The Display List Text


from wxPython.wx import wxTimer, wxFont, wxDECORATIVE, wxNORMAL
class timer(wxTimer):

	def __init__(self, parent):
		wxTimer.__init__(self)
		self.parent = parent

	def Notify(self):
		if self.parent:
			self.parent.cnt1+=0.051
			self.parent.cnt2+=0.005
			self.parent.Refresh()
		else:
			self.Stop()

class TestContext( wxInteractiveContext ):
##	rot = 0
	cnt1 = 0
	cnt2 = 0

	def __init__(self, parent):
		wxInteractiveContext.__init__(self, parent)
		f = wxFont(
			20,
			wxDECORATIVE,
			wxNORMAL,
			wxNORMAL,
			0,
			"Script",
		)
		self.SetFont( f )
		self.my_font = font( self )
		self.timer = None


	def Render( self, mode = 0):
		wxInteractiveContext.Render( self, mode )
		glDisable(GL_LIGHTING)
		glRasterPos2f(5*cos(self.cnt1), 5*sin(self.cnt2))
		self.my_font.draw("NeHe - %3.2f" % self.cnt1)
		if self.timer is None:
			self.timer = timer(self)
			self.timer.Start(25)
		
	def Background(self, mode = 0):
		''' Clear the background for a particular rendering mode,
		potentially render a "cool" background node'''
		glClearColor(0.0,0.0,1.0,1.0)
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT )

if __name__ == "__main__":
	from wxPython.wx import *
	class MyApp(wxApp):
		def OnInit(self):
			frame = wxFrame(
				NULL, -1, "NeHe GL Text Demo",
				wxDefaultPosition, wxSize(600,300),
			)
			win = TestContext(frame)
			frame.Show(TRUE)
			self.SetTopWindow(frame)
			return TRUE
	app = MyApp(0)
	app.MainLoop()
