#! /usr/bin/env python
'''Tests rendering using the ARB shader objects extension...
'''
from OpenGLContext import testingcontext
BaseContext, MainFunction = testingcontext.getInteractive()
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from OpenGLContext.arrays import array
import time, sys

def compileShader( source, shaderType ):
	"""Compile shader source of given type"""
	shader = shader_objects.glCreateShaderObjectARB(shaderType)
	shader_objects.glShaderSourceARB( shader, source )
	shader_objects.glCompileShaderARB( shader )
	return shader


def compileProgram(vertexSource=None, fragmentSource=None):
	program = shader_objects.glCreateProgramObjectARB()

	if vertexSource:
		vertexShader = compileShader(
			vertexSource, vertex_shader.GL_VERTEX_SHADER_ARB
		)
		shader_objects.glAttachObjectARB(program, vertexShader)
	if fragmentSource:
		fragmentShader = compileShader(
			fragmentSource, fragment_shader.GL_FRAGMENT_SHADER_ARB
		)
		shader_objects.glAttachObjectARB(program, fragmentShader)

	shader_objects.glValidateProgramARB( program )
	shader_objects.glLinkProgramARB(program)

	if vertexShader:
		shader_objects.glDeleteObjectARB(vertexShader)
	if fragmentShader:
		shader_objects.glDeleteObjectARB(fragmentShader)

	return program


class TestContext( BaseContext ):
	rotation = 0.00
	def Render( self, mode = 0):
		BaseContext.Render( self, mode )
		glRotate( self.rotation, 0,1,0 )
		self.rotation += .05
		shader_objects.glUseProgramObjectARB(self.program)
		glutSolidSphere(1.0,32,32)
		glTranslate( 1,0,2 )
		glutSolidCube( 1.0 )
	def OnInit( self ):
		"""Scene set up and initial processing"""
		global shader_objects, fragment_shader, vertex_shader
		shader_objects = self.extensions.initExtension( "GL.ARB.shader_objects")
		fragment_shader = self.extensions.initExtension( "GL.ARB.fragment_shader")
		vertex_shader = self.extensions.initExtension( "GL.ARB.vertex_shader" )
		if not (
			shader_objects and fragment_shader and vertex_shader
		):
			print 'Missing extension!', shader_objects, fragment_shader, vertex_shader
			sys.exit(1)
		self.program = compileProgram(
[
	'''
	varying vec3 normal;
	void main() {
		normal = gl_NormalMatrix * gl_Normal;
		gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
	}
	''',
], 
[
	'''
	varying vec3 normal;
	void main() {
		float intensity;
		vec4 color;
		vec3 n = normalize(normal);
		vec3 l = normalize(gl_LightSource[0].position).xyz;
	
		// quantize to 5 steps (0, .25, .5, .75 and 1)
		intensity = (floor(dot(l, n) * 4.0) + 1.0)/4.0;
		color = vec4(intensity*1.0, intensity*0.5, intensity*0.5,
			intensity*1.0);
	
		gl_FragColor = color;
	}
	''',
]
)


if __name__ == "__main__":
	MainFunction ( TestContext)

